###########################################################################################
# Simulation 2, Unknown testing error rates, 5000 individuals, and the following three models
#
# Model 1: logit(pi) = beta0 + beta1*X1,              beta0=-3 and beta1=2 
# Model 2: logit(pi) = beta0 + beta1*X1 + beta2*X1^2, beta0=-3, beta1=2, and beta2=-0.5
# Model 3: logit(pi) = beta0 + beta1*X1 + beta2*X2,   beta0=-3, beta1=2, and beta2=-1
#
# where X1~N(0,1) and X2~Bernoulli(0.5). Pools of size c=5 are used, under MPT, DT, and AT.


#############################################################################################
# Read in needed functions Note: Set R directory to the file that conatins the necessary dlls

source("MHS_GTR_UP.txt")
source("MHS_IND_UP.txt")
source("Testing functions.txt")


###########################################################
# Specify the number of Gibbs iterates and prior parameters
GI<-5000  # Number of Gibbs iterates

###################
# Simulation settings
N<-5000                 # Number of Individuals
sims<-500               # Number of simulations
Se.t<-.95               # Sensitivity
Sp.t<-.98               # Specificity
beta.t1<-c(-3,2)        # Model 1 parameters
beta.t2<-c(-3,2,-0.5)   # Model 2 parameters
beta.t3<-c(-3,2,-1)     # Model 3 parameters
cj<-5                   # Pool size, note array testing uses cjxcj arrays

###################
# Prior Parameters
ase<-1 
bse<-1
asp<-1
bsp<-1

save<-seq(1000,GI,5)
ns<-length(save)
np1<-length(beta.t1)
np2<-length(beta.t2)
np3<-length(beta.t3)

########################################
# Saving devices for model 1
M1.dorf.be<-array(-99,c(ns,np1,sims))
M1.dorf.se<-array(-99,c(ns,1,sims))
M1.dorf.sp<-array(-99,c(ns,1,sims))

M1.array.be<-array(-99,c(ns,np1,sims))
M1.array.se<-array(-99,c(ns,1,sims))
M1.array.sp<-array(-99,c(ns,1,sims))

M1.cost.dorf<-array(-99,c(1,sims))
M1.cost.array<-array(-99,c(1,sims))


########################################
# Saving devices for model 2
M2.dorf.be<-array(-99,c(ns,np2,sims))
M2.dorf.se<-array(-99,c(ns,1,sims))
M2.dorf.sp<-array(-99,c(ns,1,sims))

M2.array.be<-array(-99,c(ns,np2,sims))
M2.array.se<-array(-99,c(ns,1,sims))
M2.array.sp<-array(-99,c(ns,1,sims))

M2.cost.dorf<-array(-99,c(1,sims))
M2.cost.array<-array(-99,c(1,sims))


########################################
# Saving devices for model 1
M3.dorf.be<-array(-99,c(ns,np3,sims))
M3.dorf.se<-array(-99,c(ns,1,sims))
M3.dorf.sp<-array(-99,c(ns,1,sims))

M3.array.be<-array(-99,c(ns,np3,sims))
M3.array.se<-array(-99,c(ns,1,sims))
M3.array.sp<-array(-99,c(ns,1,sims))

M3.cost.dorf<-array(-99,c(1,sims))
M3.cost.array<-array(-99,c(1,sims))


set.seed(12345)

for(mod in 1:3){
for(s in 1:sims){

#######################################
# Generate true statuses
if(mod==1){
X<-cbind(rep(1,N),rnorm(N,0,1))
p<-exp(X%*%beta.t1)/(1+exp(X%*%beta.t1))
Y.true<-rbinom(N,1,p)
}

if(mod==2){
X<-cbind(rep(1,N),rnorm(N,0,1))
X<-cbind(X,X[,2]^2)
p<-exp(X%*%beta.t2)/(1+exp(X%*%beta.t2))
Y.true<-rbinom(N,1,p)
}

if(mod==3){
X<-cbind(rep(1,N),rnorm(N,0,1),rbinom(N,1,0.5))
p<-exp(X%*%beta.t3)/(1+exp(X%*%beta.t3))
Y.true<-rbinom(N,1,p)
}


#################################################################
# Simulates Dorfman testing
test.res<-Dorfman.decode.same.error(Y.true,Se.t,Sp.t,cj=cj)
Z.dorf<-test.res$Z
Y.dorf<-test.res$Y

###################################################################################################################################################################
# Data from Dorfman decoding with testing error rates unknown
if(mod==1){
res<-Bayes.reg.GT(Z=Z.dorf, Y=Y.dorf, X=X, GI=GI, b0=beta.t1, na=1, ase=ase, bse=bse, asp=asp, bsp=bsp, Se=Se.t,Sp=Sp.t, est.error=TRUE, visual=FALSE)
M1.dorf.be[,,s]<-res$beta[save,]
M1.dorf.se[,,s]<-res$Se[save,]
M1.dorf.sp[,,s]<-res$Sp[save,]
M1.cost.dorf[,s]<-dim(Z.dorf)[1]
}
if(mod==2){
res<-Bayes.reg.GT(Z=Z.dorf, Y=Y.dorf, X=X, GI=GI, b0=beta.t2, na=1, ase=ase, bse=bse, asp=asp, bsp=bsp, Se=Se.t,Sp=Sp.t, est.error=TRUE, visual=FALSE)
M2.dorf.be[,,s]<-res$beta[save,]
M2.dorf.se[,,s]<-res$Se[save,]
M2.dorf.sp[,,s]<-res$Sp[save,]
M2.cost.dorf[,s]<-dim(Z.dorf)[1]
}
if(mod==3){
res<-Bayes.reg.GT(Z=Z.dorf, Y=Y.dorf, X=X, GI=GI, b0=beta.t3, na=1, ase=ase, bse=bse, asp=asp, bsp=bsp, Se=Se.t,Sp=Sp.t, est.error=TRUE, visual=FALSE)
M3.dorf.be[,,s]<-res$beta[save,]
M3.dorf.se[,,s]<-res$Se[save,]
M3.dorf.sp[,,s]<-res$Sp[save,]
M3.cost.dorf[,s]<-dim(Z.dorf)[1]
}

#########################################################################################################################################################
# Simulates array testing  
test.res<-Array.decode.same.error(Y.true, Se.t, Sp.t, cj=cj)
Z.array<-test.res$Z
Y.array<-test.res$Y

###################################################################################################################################################################
# Data from Array decoding with testing error rates unknown
if(mod==1){
res<-Bayes.reg.GT(Z=Z.array, Y=Y.array, X=X, GI=GI, b0=beta.t1, na=1, ase=ase, bse=bse, asp=asp, bsp=bsp, Se=Se.t,Sp=Sp.t, est.error=TRUE, visual=FALSE)
M1.array.be[,,s]<-res$beta[save,]
M1.array.se[,,s]<-res$Se[save,]
M1.array.sp[,,s]<-res$Sp[save,]
M1.cost.array[,s]<-dim(Z.array)[1]
}
if(mod==2){
res<-Bayes.reg.GT(Z=Z.array, Y=Y.array, X=X, GI=GI, b0=beta.t2, na=1, ase=ase, bse=bse, asp=asp, bsp=bsp, Se=Se.t,Sp=Sp.t, est.error=TRUE, visual=FALSE)
M2.array.be[,,s]<-res$beta[save,]
M2.array.se[,,s]<-res$Se[save,]
M2.array.sp[,,s]<-res$Sp[save,]
M2.cost.array[,s]<-dim(Z.array)[1]
}
if(mod==3){
res<-Bayes.reg.GT(Z=Z.array, Y=Y.array, X=X, GI=GI, b0=beta.t3, na=1, ase=ase, bse=bse, asp=asp, bsp=bsp, Se=Se.t,Sp=Sp.t, est.error=TRUE, visual=FALSE)
M3.array.be[,,s]<-res$beta[save,]
M3.array.se[,,s]<-res$Se[save,]
M3.array.sp[,,s]<-res$Sp[save,]
M3.cost.array[,s]<-dim(Z.array)[1]
}

}} #End simulation




########################################################
# Number of tests expended
mean(M1.cost.dorf)
mean(M2.cost.dorf)
mean(M3.cost.dorf)

mean(M1.cost.array)
mean(M2.cost.array)
mean(M3.cost.array)


########################################################
# Summarize results for model 1

M1.point.be.dorf<-t(apply(M1.dorf.be,c(2,3),mean))
M1.point.se.dorf<-t(apply(M1.dorf.se,c(2,3),mean))
M1.point.sp.dorf<-t(apply(M1.dorf.sp,c(2,3),mean))

M1.point.be.array<-t(apply(M1.array.be,c(2,3),mean))
M1.point.se.array<-t(apply(M1.array.se,c(2,3),mean))
M1.point.sp.array<-t(apply(M1.array.sp,c(2,3),mean))

M1.sd.be.dorf<-t(apply(M1.dorf.be,c(2,3),sd))
M1.sd.se.dorf<-t(apply(M1.dorf.se,c(2,3),sd))
M1.sd.sp.dorf<-t(apply(M1.dorf.sp,c(2,3),sd))

M1.sd.be.array<-t(apply(M1.array.be,c(2,3),sd))
M1.sd.se.array<-t(apply(M1.array.se,c(2,3),sd))
M1.sd.sp.array<-t(apply(M1.array.sp,c(2,3),sd))

round(apply(M1.point.be.dorf,2,mean)-beta.t1,2)
round(apply(M1.point.be.dorf,2,sd),2)
round(apply(M1.sd.be.dorf,2,mean),2)
cov.func(M1.dorf.be,sims,2,0.05,beta.t1)

round(apply(M1.point.se.dorf,2,mean)-Se.t,2)
round(apply(M1.point.se.dorf,2,sd),2)
round(apply(M1.sd.se.dorf,2,mean),2)
cov.func(M1.dorf.se,sims,1,0.05,Se.t)

round(apply(M1.point.sp.dorf,2,mean)-Sp.t,2)
round(apply(M1.point.sp.dorf,2,sd),2)
round(apply(M1.sd.sp.dorf,2,mean),2)
cov.func(M1.dorf.sp,sims,1,0.05,Sp.t)

round(apply(M1.point.be.array,2,mean)-beta.t1,2)
round(apply(M1.point.be.array,2,sd),2)
round(apply(M1.sd.be.array,2,mean),2)
cov.func(M1.array.be,sims,2,0.05,beta.t1)

round(apply(M1.point.se.array,2,mean)-Se.t,2)
round(apply(M1.point.se.array,2,sd),2)
round(apply(M1.sd.se.array,2,mean),2)
cov.func(M1.array.se,sims,1,0.05,Se.t)

round(apply(M1.point.sp.array,2,mean)-Sp.t,2)
round(apply(M1.point.sp.array,2,sd),2)
round(apply(M1.sd.sp.array,2,mean),2)
cov.func(M1.array.sp,sims,1,0.05,Sp.t)



########################################################
# Summarize results for model 2

M2.point.be.dorf<-t(apply(M2.dorf.be,c(2,3),mean))
M2.point.se.dorf<-t(apply(M2.dorf.se,c(2,3),mean))
M2.point.sp.dorf<-t(apply(M2.dorf.sp,c(2,3),mean))

M2.point.be.array<-t(apply(M2.array.be,c(2,3),mean))
M2.point.se.array<-t(apply(M2.array.se,c(2,3),mean))
M2.point.sp.array<-t(apply(M2.array.sp,c(2,3),mean))

M2.sd.be.dorf<-t(apply(M2.dorf.be,c(2,3),sd))
M2.sd.se.dorf<-t(apply(M2.dorf.se,c(2,3),sd))
M2.sd.sp.dorf<-t(apply(M2.dorf.sp,c(2,3),sd))

M2.sd.be.array<-t(apply(M2.array.be,c(2,3),sd))
M2.sd.se.array<-t(apply(M2.array.se,c(2,3),sd))
M2.sd.sp.array<-t(apply(M2.array.sp,c(2,3),sd))


round(apply(M2.point.be.dorf,2,mean)-beta.t2,2)
round(apply(M2.point.be.dorf,2,sd),2)
round(apply(M2.sd.be.dorf,2,mean),2)
cov.func(M2.dorf.be,sims,3,0.05,beta.t2)

round(apply(M2.point.se.dorf,2,mean)-Se.t,2)
round(apply(M2.point.se.dorf,2,sd),2)
round(apply(M2.sd.se.dorf,2,mean),2)
cov.func(M2.dorf.se,sims,1,0.05,Se.t)

round(apply(M2.point.sp.dorf,2,mean)-Sp.t,2)
round(apply(M2.point.sp.dorf,2,sd),2)
round(apply(M2.sd.sp.dorf,2,mean),2)
cov.func(M2.dorf.sp,sims,1,0.05,Sp.t)

round(apply(M2.point.be.array,2,mean)-beta.t2,2)
round(apply(M2.point.be.array,2,sd),2)
round(apply(M2.sd.be.array,2,mean),2)
cov.func(M2.array.be,sims,3,0.05,beta.t2)

round(apply(M2.point.se.array,2,mean)-Se.t,2)
round(apply(M2.point.se.array,2,sd),2)
round(apply(M2.sd.se.array,2,mean),2)
cov.func(M2.array.se,sims,1,0.05,Se.t)

round(apply(M2.point.sp.array,2,mean)-Sp.t,2)
round(apply(M2.point.sp.array,2,sd),2)
round(apply(M2.sd.sp.array,2,mean),2)
cov.func(M2.array.sp,sims,1,0.05,Sp.t)



########################################################
# Summarize results for model 3

M3.point.be.dorf<-t(apply(M3.dorf.be,c(2,3),mean))
M3.point.se.dorf<-t(apply(M3.dorf.se,c(2,3),mean))
M3.point.sp.dorf<-t(apply(M3.dorf.sp,c(2,3),mean))

M3.point.be.array<-t(apply(M3.array.be,c(2,3),mean))
M3.point.se.array<-t(apply(M3.array.se,c(2,3),mean))
M3.point.sp.array<-t(apply(M3.array.sp,c(2,3),mean))

M3.sd.be.dorf<-t(apply(M3.dorf.be,c(2,3),sd))
M3.sd.se.dorf<-t(apply(M3.dorf.se,c(2,3),sd))
M3.sd.sp.dorf<-t(apply(M3.dorf.sp,c(2,3),sd))

M3.sd.be.array<-t(apply(M3.array.be,c(2,3),sd))
M3.sd.se.array<-t(apply(M3.array.se,c(2,3),sd))
M3.sd.sp.array<-t(apply(M3.array.sp,c(2,3),sd))

round(apply(M3.point.be.dorf,2,mean)-beta.t3,2)
round(apply(M3.point.be.dorf,2,sd),2)
round(apply(M3.sd.be.dorf,2,mean),2)
cov.func(M3.dorf.be,sims,3,0.05,beta.t3)

round(apply(M3.point.se.dorf,2,mean)-Se.t,2)
round(apply(M3.point.se.dorf,2,sd),2)
round(apply(M3.sd.se.dorf,2,mean),2)
cov.func(M3.dorf.se,sims,1,0.05,Se.t)

round(apply(M3.point.sp.dorf,2,mean)-Sp.t,2)
round(apply(M3.point.sp.dorf,2,sd),2)
round(apply(M3.sd.sp.dorf,2,mean),2)
cov.func(M3.dorf.sp,sims,1,0.05,Sp.t)

round(apply(M3.point.be.array,2,mean)-beta.t3,2)
round(apply(M3.point.be.array,2,sd),2)
round(apply(M3.sd.be.array,2,mean),2)
cov.func(M3.array.be,sims,3,0.05,beta.t3)

round(apply(M3.point.se.array,2,mean)-Se.t,2)
round(apply(M3.point.se.array,2,sd),2)
round(apply(M3.sd.se.array,2,mean),2)
cov.func(M3.array.se,sims,1,0.05,Se.t)

round(apply(M3.point.sp.array,2,mean)-Sp.t,2)
round(apply(M3.point.sp.array,2,sd),2)
round(apply(M3.sd.sp.array,2,mean),2)
cov.func(M3.array.sp,sims,1,0.05,Sp.t)
































